@@ -1,6 +1,7 @@
 from .SimConnect import SimConnect, millis, DWORD
 from .RequestList import AircraftRequests, Request
 from .EventList import AircraftEvents, Event
+from .FacilitiesList import FacilitiesRequests, Facilitie
 
 
 def int_or_str(value):
@@ -10,7 +11,7 @@ def int_or_str(value):
 		return value
 
 
-__version__ = "0.4.2"
+__version__ = "0.4.3"
 VERSION = tuple(map(int_or_str, __version__.split(".")))
 
-__all__ = ["SimConnect", "Request", "Event", "millis", "DWORD", "AircraftRequests", "AircraftEvents"]
+__all__ = ["SimConnect", "Request", "Event", "millis", "DWORD", "AircraftRequests", "AircraftEvents", "FacilitiesRequests"]
